@@ -1,19 +1,21 @@
 import json
 import logging
 import os
+import time
 from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime, timedelta
 from typing import Optional, List
 
 import requests
-from flask_restful import Resource, reqparse
 
 from ezbeq.config import Config
 
+TWO_WEEKS_AGO_SECONDS = 2 * 7 * 24 * 60 * 60
+
 logger = logging.getLogger('ezbeq.catalogue')
 
 
-class Catalogue:
+class CatalogueEntry:
 
     def __init__(self, idx: str, vals: dict):
         self.idx = idx
@@ -28,7 +30,7 @@ class Catalogue:
         self.content_type = vals.get('content_type', 'film')
         self.author = vals.get('author', '')
         self.beqc_url = vals.get('catalogue_url', '')
-        self.filters = vals.get('filters', {})
+        self.filters: List[dict] = vals.get('filters', [])
         self.images = vals.get('images', [])
         self.warning = vals.get('warning', [])
         self.season = vals.get('season', '')
@@ -44,6 +46,15 @@ class Catalogue:
         self.rating = vals.get('rating', '')
         self.genres = vals.get('genres', [])
         self.altTitle = vals.get('altTitle', '')
+        self.created_at = vals.get('created_at', 0)
+        self.updated_at = vals.get('updated_at', 0)
+        now = time.time()
+        if self.created_at >= (now - TWO_WEEKS_AGO_SECONDS):
+            self.freshness = 'Fresh'
+        elif self.updated_at >= (now - TWO_WEEKS_AGO_SECONDS):
+            self.freshness = 'Updated'
+        else:
+            self.freshness = 'Stale'
         try:
             r = int(vals.get('runtime', 0))
         except:
@@ -66,6 +77,9 @@ class Catalogue:
             'audioTypes': self.audio_types,
             'contentType': self.content_type,
             'author': self.author,
+            'created_at': self.created_at,
+            'updated_at': self.updated_at,
+            'freshness': self.freshness
         }
         if self.beqc_url:
             self.for_search['beqcUrl'] = self.beqc_url
@@ -175,14 +189,17 @@ class CatalogueProvider:
         self.__catalogue = []
         self.__executor.submit(self.__reload).result(timeout=60)
 
+    def find(self, entry_id: str) -> Optional[CatalogueEntry]:
+        return next((c for c in self.catalogue if c.idx == entry_id), None)
+
     def __reload(self):
         logger.info('Reloading catalogue')
-        downloader = DatabaseDownloader(self.__catalogue_file, self.__catalogue_version_file)
+        downloader = DatabaseDownloader(self.__config.beqcatalogue_url, self.__catalogue_file, self.__catalogue_version_file)
         reload_required = downloader.run()
         if reload_required or not self.__catalogue:
             if os.path.exists(self.__catalogue_file):
                 with open(self.__catalogue_file, 'r') as infile:
-                    self.__catalogue = [Catalogue(f"{downloader.version}_{idx}", c)
+                    self.__catalogue = [CatalogueEntry(f"{downloader.version}_{idx}", c)
                                         for idx, c in enumerate(json.load(infile))]
                     self.__loaded_at = datetime.now()
                     self.__version = downloader.version
@@ -200,7 +217,7 @@ class CatalogueProvider:
         return self.__version
 
     @property
-    def catalogue(self) -> List[Catalogue]:
+    def catalogue(self) -> List[CatalogueEntry]:
         self.__executor.submit(self.__refresh_catalogue_if_stale).result(timeout=60)
         return self.__catalogue
 
@@ -213,96 +230,14 @@ class CatalogueProvider:
                 raise e
 
 
-class ContentTypes(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-
-    def get(self):
-        catalogue = self.__provider.catalogue
-        return list(sorted({c.content_type for c in catalogue}))
-
-
-class Authors(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-
-    def get(self):
-        catalogue = self.__provider.catalogue
-        return list(sorted({c.author for c in catalogue}))
-
-
-class Years(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-
-    def get(self):
-        catalogue = self.__provider.catalogue
-        return list(sorted({c.year for c in catalogue}, reverse=True))
-
-
-class AudioTypes(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-
-    def get(self):
-        catalogue = self.__provider.catalogue
-        return list(sorted({a_t for audio_types in [c.audio_types for c in catalogue] for a_t in audio_types}))
-
-
-class CatalogueSearch(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-        self.__parser = reqparse.RequestParser()
-        self.__parser.add_argument('authors', action='append')
-        self.__parser.add_argument('years', action='append')
-        self.__parser.add_argument('audiotypes', action='append')
-        self.__parser.add_argument('contenttypes', action='append')
-        self.__parser.add_argument('fields', action='append')
-
-    def get(self):
-        catalogue = self.__provider.catalogue
-        args = self.__parser.parse_args()
-        authors = args.get('authors')
-        years = args.get('years')
-        audio_types = args.get('audiotypes')
-        content_types = args.get('contenttypes')
-        fields = args.get('fields')
-        return [self.__filter_fields(c.for_search, fields)
-                for c in catalogue if c.matches(authors, years, audio_types, content_types)]
-
-    @staticmethod
-    def __filter_fields(entry: dict, fields: list):
-        if fields:
-            return {k: v for k, v in entry.items() if k in fields}
-        else:
-            return entry
-
-
-class CatalogueMeta(Resource):
-
-    def __init__(self, **kwargs):
-        self.__provider: CatalogueProvider = kwargs['catalogue']
-
-    def get(self):
-        version = self.__provider.version
-        loaded_at = self.__provider.loaded_at
-        return {
-            'version': version,
-            'loaded': int(loaded_at.timestamp()) if loaded_at is not None else None,
-            'count': len(self.__provider.catalogue)
-        }
-
-
 class DatabaseDownloader:
-    DATABASE_CSV = 'http://beqcatalogue.readthedocs.io/en/latest/database.json'
-    VERSION_TXT = 'http://beqcatalogue.readthedocs.io/en/latest/version.txt'
 
-    def __init__(self, cached_db_file, cached_version_file):
+    def __init__(self, download_url: str, cached_db_file, cached_version_file):
+        self.__download_url = download_url
+        if download_url[-1] != '/':
+            self.__download_url = f"{download_url}/"
+        self.__db_url = f"{self.__download_url}database.json"
+        self.__version_url = f"{self.__download_url}version.txt"
         self.__cached_db_file = cached_db_file
         self.__cached_version_file = cached_version_file
         self.__cached_version = ''
@@ -322,8 +257,8 @@ class DatabaseDownloader:
         '''
         remote_version = self.__get_remote_catalogue_version()
         if remote_version is None or self.__cached_version is None or remote_version != self.__cached_version:
-            logger.info(f"Reloading from {self.DATABASE_CSV}")
-            r = requests.get(self.DATABASE_CSV, allow_redirects=True)
+            logger.info(f"Reloading from {self.__db_url}")
+            r = requests.get(self.__db_url, allow_redirects=True)
             if r.status_code == 200:
                 logger.info(f"Writing database to {self.__cached_db_file}")
                 with open(self.__cached_db_file, 'wb') as f:
@@ -335,7 +270,7 @@ class DatabaseDownloader:
                 else:
                     logger.warning(f"No remote version to write")
                 self.__cached_version = remote_version
-                logger.info(f"Downloaded {self.DATABASE_CSV} @ {remote_version}")
+                logger.info(f"Downloaded {self.__db_url} @ {remote_version}")
                 return True
             else:
                 logger.warning(f"Unable to download catalogue, response is {r.status_code}")
@@ -349,11 +284,11 @@ class DatabaseDownloader:
         :return: the version, if any.
         '''
         try:
-            r = requests.get(self.VERSION_TXT, allow_redirects=True)
+            r = requests.get(self.__version_url, allow_redirects=True)
             if r.status_code == 200:
                 return r.text
             else:
-                logger.warning(f"Unable to get {self.VERSION_TXT}, response was {r.status_code}")
+                logger.warning(f"Unable to get {self.__version_url}, response was {r.status_code}")
         except:
-            logger.exception(f"Failed to get {self.VERSION_TXT}")
+            logger.exception(f"Failed to get {self.__version_url}")
         return None
