@@ -1,3 +1,4 @@
+import json
 import logging
 import os
 from concurrent.futures.thread import ThreadPoolExecutor
@@ -177,8 +178,10 @@ class Minidsp(PersistentDevice[MinidspState]):
         self.__executor = ThreadPoolExecutor(max_workers=1)
         self.__cmd_timeout = cfg.get('cmdTimeout', 10)
         self.__ignore_retcode = cfg.get('ignoreRetcode', False)
+        self.__levels_interval = 1.0 / float(cfg.get('levelsFps', 10))
         self.__runner = cfg['make_runner']()
         self.__client = MinidspRsClient(self) if cfg.get('useWs', False) else None
+        ws_server.factory.set_levels_provider(name, self.start_broadcast_levels)
 
     @property
     def device_type(self) -> str:
@@ -193,23 +196,16 @@ class Minidsp(PersistentDevice[MinidspState]):
         return result if result else MinidspState(self.name)
 
     def __read_state_from_device(self) -> Optional[MinidspState]:
-        values = {}
         lines = None
         try:
             kwargs = {'retcode': None} if self.__ignore_retcode else {}
-            lines = self.__runner(timeout=self.__cmd_timeout, **kwargs)
-            for line in lines.split('\n'):
-                if line.startswith('MasterStatus'):
-                    idx = line.find('{ ')
-                    if idx > -1:
-                        vals = line[idx + 2:-2].split(', ')
-                        for v in vals:
-                            if v.startswith('preset: '):
-                                values['active_slot'] = str(int(v[8:]) + 1)
-                            elif v.startswith('mute: '):
-                                values['mute'] = v[6:] == 'true'
-                            elif v.startswith('volume: Gain('):
-                                values['mv'] = float(v[13:-1])
+            output = self.__runner['-o', 'jsonline'](timeout=self.__cmd_timeout, **kwargs)
+            status = json.loads(output)
+            values = {
+                'active_slot': str(status['master']['preset'] + 1),
+                'mute': status['master']['mute'],
+                'mv': status['master']['volume']
+            }
             return MinidspState(self.name, **values)
         except:
             logger.exception(f"Unable to parse device state {lines}")
@@ -376,6 +372,39 @@ class Minidsp(PersistentDevice[MinidspState]):
             any_update = True
         return any_update
 
+    def levels(self) -> dict:
+        return self.__executor.submit(self.__read_levels_from_device).result(timeout=self.__cmd_timeout)
+
+    def __read_levels_from_device(self) -> dict:
+        lines = None
+        try:
+            kwargs = {'retcode': None} if self.__ignore_retcode else {}
+            start = time.time()
+            lines = self.__runner['-o', 'jsonline'](timeout=self.__cmd_timeout, **kwargs)
+            end = time.time()
+            levels = json.loads(lines)
+            ts = time.time()
+            logger.info(f"readlevels,{ts},{to_millis(start, end)}")
+            return {
+                'ts': ts,
+                'input': levels['input_levels'],
+                'output': levels['output_levels']
+            }
+        except:
+            logger.exception(f"Unable to load levels {lines}")
+            return {}
+
+    def start_broadcast_levels(self) -> None:
+        from twisted.internet import reactor
+        sched = lambda: reactor.callLater(self.__levels_interval, __send)
+
+        def __send():
+            msg = json.dumps(self.levels())
+            if self.ws_server.levels(self.name, msg):
+                sched()
+
+        sched()
+
 
 class MinidspBeqCommandGenerator:
 
